'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.argon2d = exports.argon2i = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ref = require('ref');

var _ref2 = _interopRequireDefault(_ref);

var _library = require('./library');

var _library2 = _interopRequireDefault(_library);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var argon2ErrorMessage = _library2.default.argon2_error_message;

var ARGON2_OK = 0;

var defaultOptions = {
  timeCost: 3,
  memoryCost: 1 << 12,
  parallelism: 1,
  hashLength: 32
};

function parseArgs(args) {
  var password = void 0;
  var salt = void 0;
  var options = {};
  var cb = void 0;

  if (args.length === 3) {
    var _args = _slicedToArray(args, 3);

    password = _args[0];
    salt = _args[1];
    cb = _args[2];
  } else {
    var _args2 = _slicedToArray(args, 4);

    password = _args2[0];
    salt = _args2[1];
    options = _args2[2];
    cb = _args2[3];
  }

  var parsedPassword = password;
  if (typeof password === 'string') {
    parsedPassword = new Buffer(password);
  }

  var hashOptions = Object.assign({}, defaultOptions, options);
  return [parsedPassword, salt, hashOptions, cb];
}

function variant(_hashRaw, hashEncoded, _verify) {
  return {
    hashRaw: function hashRaw() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var _parseArgs = parseArgs(args);

      var _parseArgs2 = _slicedToArray(_parseArgs, 4);

      var password = _parseArgs2[0];
      var salt = _parseArgs2[1];
      var options = _parseArgs2[2];
      var cb = _parseArgs2[3];
      var timeCost = options.timeCost;
      var memoryCost = options.memoryCost;
      var parallelism = options.parallelism;
      var hashLength = options.hashLength;

      var hashOutput = new Buffer(hashLength);
      var resultHandler = function resultHandler(err, res) {
        if (err) {
          return cb(err, null);
        }
        if (res !== ARGON2_OK) {
          return cb(new Error(argon2ErrorMessage(res)), null);
        }
        return cb(null, hashOutput);
      };
      _hashRaw.async(timeCost, memoryCost, parallelism, password, password.length, salt, salt.length, hashOutput, hashLength, resultHandler);
      return hashOutput;
    },
    hash: function hash() {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      var _parseArgs3 = parseArgs(args);

      var _parseArgs4 = _slicedToArray(_parseArgs3, 4);

      var password = _parseArgs4[0];
      var salt = _parseArgs4[1];
      var options = _parseArgs4[2];
      var cb = _parseArgs4[3];
      var timeCost = options.timeCost;
      var memoryCost = options.memoryCost;
      var parallelism = options.parallelism;
      var hashLength = options.hashLength;

      var encodedSize = _library2.default.argon2_encodedlen(timeCost, memoryCost, parallelism, salt.length, hashLength);
      var outputBuffer = new Buffer(encodedSize);
      var resultHandler = function resultHandler(err, res) {
        if (err) {
          return cb(err, null);
        }
        if (res !== ARGON2_OK) {
          return cb(new Error(argon2ErrorMessage(res)), null);
        }
        return cb(null, _ref2.default.readCString(outputBuffer, 0));
      };
      hashEncoded.async(timeCost, memoryCost, parallelism, password, password.length, salt, salt.length, hashLength, outputBuffer, outputBuffer.length, resultHandler);
    },
    verify: function verify(encoded, password, cb) {
      var encodedBuffer = _ref2.default.allocCString(encoded || '');
      var parsedPassword = password;
      if (typeof password === 'string') {
        parsedPassword = new Buffer(password);
      }
      _verify.async(encodedBuffer, parsedPassword, parsedPassword.length, function (err, res) {
        if (err) {
          return cb(err, null);
        }
        if (res !== ARGON2_OK) {
          return cb(new Error(argon2ErrorMessage(res)), null);
        }
        return cb(null, res);
      });
    }
  };
}

var argon2i = exports.argon2i = variant(_library2.default.argon2i_hash_raw, _library2.default.argon2i_hash_encoded, _library2.default.argon2i_verify);
var argon2d = exports.argon2d = variant(_library2.default.argon2d_hash_raw, _library2.default.argon2d_hash_encoded, _library2.default.argon2d_verify);